<?php
/* export/agi_export.php
.---------------------------------------------------------------------------.
|    Software: agi_export for modified eCommerce                            |
|      Author: Andreas Guder                                                |
|     Contact: info@andreas-guder.de                                        |
| Copyright (c) 2022, Andreas Guder [info@andreas-guder.de]                 |
|               All Rights Reserved.                                        |
| ------------------------------------------------------------------------- |
|     License: you are not allowed to distribute                            |
|              or sublicense copies of the software without                 |
|              a grant of a licence of the coyright-owner                   |
'---------------------------------------------------------------------------ü
*/

require('../includes/application_top_export.php');
foreach(auto_include(DIR_FS_CATALOG.'includes/extra/application_top/application_top_begin/','php') as $file) require_once ($file);

require_once (DIR_FS_INC.'xtc_get_products_mo_images.inc.php');
require_once (DIR_FS_INC.'xtc_parse_category_path.inc.php');
require_once (DIR_FS_INC.'xtc_get_product_path.inc.php');
require_once (DIR_FS_INC.'xtc_get_tax_rate.inc.php');
require_once (DIR_FS_INC.'xtc_href_link.inc.php');
require_once (DIR_FS_INC.'xtc_product_link.inc.php');
require_once (DIR_FS_INC.'xtc_not_null.inc.php');
require_once (DIR_FS_INC.'xtc_get_category_path.inc.php');
require_once (DIR_FS_INC.'xtc_get_parent_categories.inc.php');
require_once (DIR_FS_INC.'xtc_get_all_get_params.inc.php');

if (!defined('MODULE_SYSTEM_AGI_EXPORT_STATUS') || (defined('MODULE_SYSTEM_AGI_EXPORT_STATUS') && MODULE_SYSTEM_AGI_EXPORT_STATUS != 'true')) {
  header("HTTP/1.0 406 Not Acceptable");
  exit;
}

$_croncall = isset($_GET['croncall']);
$_feed = isset($_GET['feed']) ? (int)$_GET['feed'] : 0;
if (empty($_feed)) {
  header("HTTP/1.0 406 Not Acceptable - parameter required");
  exit;
}

// load agi_export config
$_agi_export_config = array();
$qr = xtc_db_query("SELECT `config_key`, IF (`configured`=1, `config_value`, `preset`) AS conf_value FROM agi_export_config");
while ($row = xtc_db_fetch_array($qr)) {
  $chr = '';
  if (!empty($row['conf_value'])) {
    $chr = substr($row['conf_value'], 0, 1);
  }
  $_agi_export_config[$row['config_key']] = in_array($chr, array('[','{')) ? json_decode($row['conf_value'], true) : $row['conf_value'];
}
if (isset($_GET['show_version'])) {
  die($_agi_export_config['version']);
}
define('AGI_EXPORT_QUICK', isset($_GET['quick']));

if (!empty($_agi_export_config['feed_password']) && (!isset($_GET['pass']) || (isset($_GET['pass']) && $_GET['pass'] != $_agi_export_config['feed_password']))) {
  header("HTTP/1.0 403 Forbidden");
  exit;
}

// load feeds-data
$qr = xtc_db_query("SELECT *, DATE_SUB(NOW(), INTERVAL 30 MINUTE) as cron_max FROM `agi_export_feeds` WHERE `id`=".$_feed." AND `status`=1");
if (!xtc_db_num_rows($qr)) {
  header("HTTP/1.0 404 Not Found");
  exit;
}

$_feed_config = xtc_db_fetch_array($qr);
$_feed_file_base = DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_feed_config['id'];
$_quick = '';

$products_conditions_p = GROUP_CHECK == 'true' ? ' AND p.group_permission_'.$_feed_config['customers_group'].' = 1 ' : '';
$categories_conditions_c = GROUP_CHECK == 'true' ? ' AND c.group_permission_'.$_feed_config['customers_group'].' = 1 ' : '';
defined('PRODUCTS_CONDITIONS_P') OR define('PRODUCTS_CONDITIONS_P', $products_conditions_p);
defined('PRODUCTS_CONDITIONS') OR define('PRODUCTS_CONDITIONS', str_replace(' p.', ' ', $products_conditions_p));
defined('CATEGORIES_CONDITIONS_C') OR define('CATEGORIES_CONDITIONS_C', $categories_conditions_c);
defined('CATEGORIES_CONDITIONS') OR define('CATEGORIES_CONDITIONS', str_replace(' c.', ' ', $categories_conditions_c));


if (AGI_EXPORT_QUICK) {
  $_feed_file_base .= '.quick';
  $_quick = 'quick_';
}
if (array_key_exists('products_filter', $_feed_config) && !empty($_feed_config['products_filter'])) {
  if (substr($_feed_config['products_filter'], 0, 1) == '{') {
    $_feed_config['products_filter'] = json_decode($_feed_config['products_filter'], true);
  } else {
    $_feed_config['products_filter'] = array();
  }
} else {
  $_feed_config['products_filter'] = array();
}

require_once (DIR_FS_EXTERNAL.'agi/agi_export/export/classes/agi_shop_config.class.php');
if ($_feed_config['type'] == 'google_xml') {
  require_once (DIR_FS_EXTERNAL.'agi/agi_export/export/classes/xml_feed_builder.class.php');
  class_alias('XML_FEED_BUILDER', 'AGI_FEED_BUILDER', false);
} else {
  require_once (DIR_FS_EXTERNAL.'agi/agi_export/export/classes/csv_feed_builder.class.php');
  class_alias('CSV_FEED_BUILDER', 'AGI_FEED_BUILDER', false);
}
require_once (DIR_FS_EXTERNAL.'agi/agi_export/export/classes/agi_product_feed.class.php');

if (!$_croncall) {
  if (!file_exists($_feed_file_base.'.final')) {
    if ($_feed_config['cron']) {
      header("HTTP/1.0 204 No Content - come back later");
      exit;
    }
  } else {
    $cached_filetime = filemtime($_feed_file_base.'.final');
    if (time()-$cached_filetime < (24 * 3600 - 3600) || $_feed_config['cron']) {
      if ($_feed_config['type'] == 'google_xml') {
        // print out cached file
        header("Content-Type: text/xml");
        // print out header
        echo '<?xml version="1.0" encoding="UTF-8"?>',chr(10);
        echo '<rss xmlns:g="http://base.google.com/ns/1.0" version="2.0">',chr(10);
        echo '<channel>',chr(10);
        echo '<title>XML-Datenfeed</title>',chr(10);
        echo '<link>',HTTP_SERVER,'</link>',chr(10);
        echo '<description>Google product feed created at ',date('Y-m-d H:i', $cached_filetime),'</description>',chr(10);
        readfile($_feed_file_base.'.final');
        // print out footer
        echo '</channel>',chr(10);
        echo '</rss>';
      }
      exit;
    }
  }
} else {
  if ($_feed_config['cron']) {
    if ($_feed_config[$_quick.'in_process'] != '0000-00-00 00:00:00') {
      if ($_feed_config[$_quick.'in_process'] < $_feed_config['cron_max']) {
        // reset cron
        xtc_db_perform('agi_export_feeds', array($_quick.'in_process' => '0000-00-00 00:00:00', $_quick.'cron_error_cnt'=>$_feed_config[$_quick.'cron_error_cnt']+1), 'update', "`id`=".$_feed);
        header("HTTP/1.0 202 Accepted");
        echo 'Cron Reset';
        exit;
      } else {
        // allways runs
        header("HTTP/1.0 202 Accepted");
        echo 'Runs';
        exit;
      }
    }
  } else {
    header("HTTP/1.0 405 Method Not Allowed");
    echo 'Its a Cron-Call';
    exit;
  }
}

session_start();
$_SESSION['languages_id'] = $_feed_config['language'];

function agi_export_get_id_string_from($id_string) {
  if (empty($id_string)) {
    return '';
  }
  $a_tmp = explode(',', $id_string);
  $ids = array();
  foreach($a_tmp as $value) {
    $t = preg_replace('/\D/', '', $value);
    if (!empty($t)) {
      $ids[] = $t;
    }
  }
  $ids = array_unique($ids);
  return !empty($ids) ? implode(',', $ids) : '';
}

$config = new AGI_SHOP_CONFIG($_agi_export_config, $_feed_config);
$_SESSION['language_code'] = $config::get_language_code();

// Prepare general database query
$general_query = "SELECT 
    p.*, pd.*,
    IFNULL(s.status, 0) AS product_has_special,
    IFNULL(s.specials_new_products_price, 0) AS specials_new_products_price,
    IFNULL(s.expires_date,'') AS special_expires_date,
    IFNULL(s.start_date,'') AS special_start_date
  FROM 
    `".TABLE_PRODUCTS."` AS p
    LEFT JOIN `".TABLE_PRODUCTS_DESCRIPTION."` AS pd ON pd.products_id=p.products_id AND pd.language_id=".AGI_SHOP_CONFIG::get_language_id()."
    LEFT JOIN `".TABLE_SPECIALS."` AS s ON s.products_id=p.products_id
  WHERE 
    p.products_status=1 AND p.agi_export_feed_".$_feed_config['id']."=1";
$total_check_query = "SELECT COUNT(*) AS cnt FROM ".TABLE_PRODUCTS." WHERE products_status=1 AND agi_export_feed_".$_feed_config['id']."=1";

if (strtolower(GROUP_CHECK) == 'true') {
  $general_query .= " AND p.group_permission_".AGI_SHOP_CONFIG::get_customers_group()."=1";
  $total_check_query .= " AND group_permission_".AGI_SHOP_CONFIG::get_customers_group()."=1";
}

if (!empty($_feed_config['products_filter']['exclude_manu'])) {
  $tmp = agi_export_get_id_string_from($_feed_config['products_filter']['exclude_manu']);
  if (!empty($tmp)) {
    $general_query .= " AND p.manufacturers_id NOT IN ($tmp)";
    $total_check_query .= " AND manufacturers_id NOT IN ($tmp)";
  }
}
if (!empty($_feed_config['products_filter']['include_manu'])) {
  $tmp = agi_export_get_id_string_from($_feed_config['products_filter']['include_manu']);
  if (!empty($tmp)) {
    $general_query .= " AND p.manufacturers_id IN ($tmp)";
    $total_check_query .= " AND manufacturers_id IN ($tmp)";
  }
}

if (!empty($_feed_config['products_filter']['exclude_stimes'])) {
  $tmp = agi_export_get_id_string_from($_feed_config['products_filter']['exclude_stimes']);
  if (!empty($tmp)) {
    $general_query .= " AND p.products_shippingtime NOT IN ($tmp)";
    $total_check_query .= " AND products_shippingtime NOT IN ($tmp)";
  }
}
if (!empty($_feed_config['products_filter']['include_stimes'])) {
  $tmp = agi_export_get_id_string_from($_feed_config['products_filter']['include_stimes']);
  if (!empty($tmp)) {
    $general_query .= " AND p.products_shippingtime IN ($tmp)";
    $total_check_query .= " AND products_shippingtime IN ($tmp)";
  }
}

if (empty($_feed_config['products_filter']['fsk18'])) {
  $general_query .= " AND p.products_fsk18=0";
  $total_check_query .= " AND products_fsk18=0";
}

if (!empty($_feed_config['products_filter']['minprice'])) {
  if (preg_match('/^\d+(\.\d+)?$/', $_feed_config['products_filter']['minprice'])) {
    $general_query .= " AND p.products_price>=".$_feed_config['products_filter']['minprice'];
    $total_check_query .= " AND products_price>=".$_feed_config['products_filter']['minprice'];
  }
}
if (!empty($_feed_config['products_filter']['maxprice'])) {
  if (preg_match('/^\d+(\.\d+)?$/', $_feed_config['products_filter']['maxprice'])) {
    $general_query .= " AND p.products_price<=".$_feed_config['products_filter']['maxprice'];
    $total_check_query .= " AND products_price<=".$_feed_config['products_filter']['maxprice'];
  }
}
if (!empty($_feed_config['products_filter']['minstock'])) {
  if (preg_match('/^\-?\d+(\.\d+)?$/', $_feed_config['products_filter']['minstock'])) {
    $general_query .= " AND p.products_quantity>=".$_feed_config['products_filter']['minstock'];
    $total_check_query .= " AND products_quantity>=".$_feed_config['products_filter']['minstock'];
  }
}

$settings = array(
  'is_utf8_shop'  => $_agi_export_config['is_utf8_shop']
);
if ($_feed_config['type'] == 'csv') {
  if (!empty($_feed_config['csv_settings'])) {
    $settings = array_merge($settings, json_decode($_feed_config['csv_settings'], true));
  }
}  
$output_feed = new AGI_PRODUCT_FEED($_feed_file_base, $settings);
$output_feed->start_feed();

// start query-execution
xtc_db_query("UPDATE `agi_export_feeds` SET `".$_quick."in_process`=NOW() WHERE `id`=".$_feed);
$_start_on_pid = 0;
if ($_croncall) {
  $_start_on_pid = $_feed_config[$_quick.'cron_last_pid'];
}
$mt = @ini_get('max_execution_time');
if (empty($mt)) {
  $mt = 30;
}
if ($mt > 300) {
  $mt = 300;
}

$time_limit = time()+$mt-5;
$start_time = time();
$general_query_limit = AGI_EXPORT_QUICK ? ' AND p.products_id > %s ORDER BY p.products_id LIMIT 2000' : ' AND p.products_id > %s ORDER BY p.products_id LIMIT 200';
$general_query_block = $general_query.sprintf($general_query_limit, $_start_on_pid);

$main_product_query = xtc_db_query($general_query_block);
$_last_written_pid = 0;
$stop_on_time = false;
$cnt_call = 1;
while(xtc_db_num_rows($main_product_query)) {
  while ($listing = xtc_db_fetch_array($main_product_query)) {
    $_last_written_pid = $listing['products_id'];
    
    // Hier die Auswertung der query-results einfügen
    if (empty($listing['products_image']) && $_feed_config['type'] == 'google_xml') {
      continue;
    }
    $output_feed->set_item_data($listing);
    if ($output_feed->prepare_product()) {
      // jetzt ausgeben! nur wenn keine Attribute, da ausgabe sonst schon erledigt
      $output_feed->write_item_data();
    }
  }
  xtc_db_free_result($main_product_query);
  if (time() > $time_limit) {
    $stop_on_time = true;
    break;
  } else {
    $general_query_block = $general_query.sprintf($general_query_limit, $_last_written_pid);
    $main_product_query = xtc_db_query($general_query_block);
  }
  $cnt_call++;
}
if ($stop_on_time) {
  // wenn cron, dann update cron
  if (!$_feed_config['cron']) {
    // Fehler, feed nicht fertig
    xtc_db_perform('agi_export_feeds', array($_quick.'cron_last_pid'=>0, $_quick.'in_process'=>'0000-00-00 00:00:00', 'clear_after_process'=>0), 'update', "`id`=".$_feed);
    header("HTTP/1.0 200 Request Timeout");
    echo 'Ausf&uuml;hrungszeit zur kurz, Feed kann nicht vollständig erstellt werden, nutzen Sie den Cronjob-Aufruf!';
    exit;
  } else {
    $qr = xtc_db_query("SELECT `clear_after_process` FROM `agi_export_feeds` WHERE `id`=".$_feed);
    $tmp = xtc_db_fetch_array($qr);
    $_feed_config['clear_after_process'] = $tmp['clear_after_process'];
    if ($_feed_config['clear_after_process']) {
      $_last_written_pid = 0;
      if (file_exists(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.tmp')) {
        @unlink(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.tmp');
      }
      if (file_exists(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.quick.final')) {
        @unlink(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.quick.tmp');
      }
    }
    xtc_db_perform('agi_export_feeds', array($_quick.'cron_last_pid'=>$_last_written_pid, $_quick.'in_process'=>'0000-00-00 00:00:00','clear_after_process'=>0), 'update', "`id`=".$_feed);
    header("HTTP/1.0 206 Partial Content");
    echo 'Next cronrun needed';
    exit;
  }
} else {
  //realy finished?
  $total_check = xtc_db_query($total_check_query." AND products_id>$_last_written_pid");
  $total_check_result = xtc_db_fetch_array($total_check);
  if ($total_check_result && $total_check_result['cnt'] > 0) {
    // not finished yet, so start again
    if (!$_feed_config['cron']) {
      // can not start again - wait for next call if file exists
      xtc_db_perform('agi_export_feeds', array($_quick.'cron_last_pid'=>0, $_quick.'in_process'=>'0000-00-00 00:00:00', 'clear_after_process'=>0), 'update', "`id`=".$_feed);
      if (!file_exists($_feed_file_base.'.final')) {
        header("HTTP/1.0 200 Request Timeout");
        echo 'SQl-Fehler oder Ähnliches Problem aufgetreten, der Feed wurde nicht erstellt. Bitte URL erneut aufrufen.';
        exit;
      }
    } else {
      $qr = xtc_db_query("SELECT `clear_after_process` FROM `agi_export_feeds` WHERE `id`=".$_feed);
      $tmp = xtc_db_fetch_array($qr);
      $_feed_config['clear_after_process'] = $tmp['clear_after_process'];
      if ($_feed_config['clear_after_process']) {
        $_last_written_pid = 0;
        if (file_exists(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.tmp')) {
          @unlink(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.tmp');
        }
        if (file_exists(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.quick.final')) {
          @unlink(DIR_FS_EXTERNAL.'agi/agi_export/feeds/feed_'.$_GET['set'].'.quick.tmp');
        }
      }
      xtc_db_perform('agi_export_feeds', array($_quick.'cron_last_pid'=>$_last_written_pid, $_quick.'in_process'=>'0000-00-00 00:00:00','clear_after_process'=>0), 'update', "`id`=".$_feed);
      header("HTTP/1.0 206 Partial Content");
      echo 'Next cronrun needed';
      exit;
    }
    
  } else {
    xtc_db_perform('agi_export_feeds', array($_quick.'cron_last_pid'=>0, $_quick.'in_process'=>'0000-00-00 00:00:00', $_quick.'cron_error_cnt'=>0,'clear_after_process'=>0), 'update', "`id`=".$_feed);
    copy($_feed_file_base.'.tmp', $_feed_file_base.'.final');
    unlink($_feed_file_base.'.tmp');
  }
}
$output_feed->end_feed();
if ($_croncall) {
  header("HTTP/1.0 201 Created");
  echo 'Cron finished feed';
  exit;
}

if ($_feed_config['type'] == 'google_xml') {
  // print out cached file
  header("Content-Type: text/xml");
  // print out header
  echo '<?xml version="1.0" encoding="UTF-8"?>',chr(10);
  echo '<rss xmlns:g="http://base.google.com/ns/1.0" version="2.0">',chr(10);
  echo '<channel>',chr(10);
  echo '<title>XML-Datenfeed</title>',chr(10);
  echo '<link>',HTTP_SERVER,'</link>',chr(10);
  echo '<description>Google product feed created at ',date('Y-m-d H:i'),'</description>',chr(10);
  readfile($_feed_file_base.'.final');
  // print out footer
  echo '</channel>',chr(10);
  echo '</rss>';
}
exit;
