<?php
/* includes/external/agi/agi_export/export/classes/agi_product_feed.class.php
.---------------------------------------------------------------------------.
|    Software: agi_export for modified eCommerce                            |
|      Author: Andreas Guder                                                |
|     Contact: info@andreas-guder.de                                        |
| Copyright (c) 2022, Andreas Guder [info@andreas-guder.de]                 |
|               All Rights Reserved.                                        |
| ------------------------------------------------------------------------- |
|     License: you are not allowed to distribute                            |
|              or sublicense copies of the software without                 |
|              a grant of a licence of the coyright-owner                   |
'---------------------------------------------------------------------------ü
*/

class AGI_PRODUCT_FEED extends AGI_FEED_BUILDER
{
  private $products_id = 0;
  private $products_discount = 0;
  public $product_data_raw = array();
  public $product_data_use = array();
  private $product_images = array();
  private $product_attribute_images = array();
  private $cartesian_attributes = array();
  
  private $overwrite_attributes = array();
  private $my_google_attribute_options_ids = array();
  private $my_google_attribute_options_ids_name = array();
  private $max_overwrite_attributes = 1;
  private $my_attributes_options = array(); // AGI Attribut-Link
  
  public function __construct($_feed_file_base, $settings = array())
  {
    parent::__construct($_feed_file_base, $settings);
  }
  
  public function set_item_data($product_data)
  {
    $this->products_id = (int) $product_data['products_id'];
    $this->product_data_raw = $product_data;
    $this->product_data_use = $product_data;
    
    $this->overwrite_attributes = array();
    $this->my_google_attribute_options_ids = array();
    $this->my_google_attribute_options_ids_name = array();
    $this->clear_item_data();
  }
  
  public function clean_html_to_text($text, $short = 0)
  {
    $find     = array('</p>', '<br />', '<li />', '>');
    $replace  = array('</p>'.chr(10), '<br />'.chr(10), '</li>'.chr(10), '> ');
    $text = str_replace($find, $replace, $text);
    $cleared_text = strip_tags($text);
    $cleared_text = str_replace("&nbsp;"," ",$cleared_text);
    $cleared_text = html_entity_decode($cleared_text,ENT_QUOTES, AGI_SHOP_CONFIG::is_utf8_shop() ? 'UTF-8' : $_SESSION['language_charset']);
    $cleared_text = str_replace("'",", ",$cleared_text);
    $cleared_text = str_replace("\n"," ",$cleared_text);
    $cleared_text = str_replace("\r"," ",$cleared_text);
    $cleared_text = str_replace("\t"," ",$cleared_text);
    $cleared_text = str_replace("\v"," ",$cleared_text);
    $cleared_text = str_replace("|",",",$cleared_text);
    $cleared_text = preg_replace("/\s{2,}/"," ",$cleared_text);
    $cleared_text = preg_replace("/[[:cntrl:]]+/", '',$cleared_text);
    if ($short > 0) {
      $cleared_text = $this->short_string($cleared_text, $short);
    }
    
    $cleared_text = trim($cleared_text);
    return $cleared_text;
  }
  
  private function print_out_product()
  {
    $this->add_feed_item();
    $this->clear_item_data();
    $this->product_images = array();
  }
  
  /**
   * get a cartesian_product of the given attribute_array
   */
  private function get_cartesian_array($sofar,$arr,$pos,$max)
  {
    $tmp = array_keys($arr);
    for ($i = 0; $i < count($arr[$tmp[$pos]]); $i++) {
      if($pos == $max) {
        $this->cartesian_attributes[] = array_merge($sofar, array($arr[$tmp[$pos]][$i]));
      }
      else {
        $this->get_cartesian_array(array_merge($sofar, array($arr[$tmp[$pos]][$i])), $arr, $pos+1, $max);
      }
    }
  }
  
  /**
   * combines all product-attributes (cartesian)
   * @param array
   * @return array
   */
  public function combine_attributes($arr)
  {
    $this->cartesian_attributes = array();
    $this->get_cartesian_array(array(), $arr, 0, count($arr)-1);
    return $this->cartesian_attributes;
  }
  
  /**
   * return shipping-cost calculated by given shipping-table
   * mod for Shipping to more countries
   * @param double products_price
   * @param double products_weight
   * @return double
   */
  public function calculate_shipping_from_table($products_price, $products_weight, $country = '')
  {
    $shipping = -1;
    $base = AGI_SHOP_CONFIG::get_agi_export_conf('shipping_base') == 'weight' ? $products_weight : $products_price;
    if (AGI_SHOP_CONFIG::get_agi_export_conf('shipping_base') == 'weight' && defined('SHIPPING_BOX_WEIGHT')) {
      $base += SHIPPING_BOX_WEIGHT;
    }
    $shipping_value_table = AGI_SHOP_CONFIG::get_shipping_value_table($country);
    for ($i=0; $i<sizeof($shipping_value_table); $i+=2) {
      if ($base <= $shipping_value_table[$i]) {
        $shipping = $shipping_value_table[$i+1];
        break;
      }
    }
    if ($shipping == -1) {
      $shipping = 0;
    }
    return $shipping;
  }
  
  // mod for Shipping to more countries
  public function calculate_shipping($country = '')
  {
    // calculate shipping
    $s_price = $this->product_data_use['products_price'];
    if ($this->product_data_use['product_has_special']) { 
      $special_start = !empty($this->product_data_use['special_start_date']) ? strtotime($this->product_data_use['special_start_date']) : time();
      if($special_start <= time()) {
        $s_price = $this->product_data_use['specials_new_products_price'];
      }
    }
    $shipping = 0;
    if (AGI_SHOP_CONFIG::get_agi_export_conf('shipping_ignore_freeshipping')) {
      $shipping = $this->calculate_shipping_from_table($s_price, $this->product_data_use['products_weight'], $country);
    }
    else {
      if (empty($country) && AGI_SHOP_CONFIG::is_active_free_amount_module() && $s_price >= AGI_SHOP_CONFIG::get_free_amount_value()) {
        $shipping = 0;
      }
      else {
        $shipping = $this->calculate_shipping_from_table($s_price, $this->product_data_use['products_weight'], $country);
      }
    }
    if (!empty($country)) {
      $this->product_data_use['shipping_price_'.$country] = $shipping;
    }
    else {
      $this->product_data_use['shipping_price'] = $shipping;
    }
  }
  
  /* BOF Shipping to more countries */
  public function calculate_morecountry_shipping()
  {
    $list = AGI_SHOP_CONFIG::get_country_shipping_list();
    if (!empty($list)) {
      foreach($list as $country) {
        $this->calculate_shipping($country);
      }
    }
  }
  
  /* EOF Shipping to more countries */
  public function prepare_product()
  {
    $this->check_special();
    if ($this->set_personal_price() <= 0) {
      return false;
    }
    if (!AGI_EXPORT_QUICK) {
      $this->get_category();
      $this->create_link();
      $this->set_brand();
      $this->set_category();
      $this->set_images();
    }
    $this->set_vpe();
    $this->set_availability();
    if (!AGI_EXPORT_QUICK) {
      $this->set_products_texts();
    }
    $this->calculate_shipping();
    $this->calculate_morecountry_shipping();
    $this->prepare_attributes();
    if (AGI_SHOP_CONFIG::get_agi_feed_conf('export_attributes') && $this->attribute_handling()) {
      // returns true, if article has attributes, musnt print out later
      return false;
    }
    return true;
  }
  
  private function get_category()
  {
    $categories = 0;
    $this->product_data_raw['categories_id'] = 0;
    $categorie_query=xtDBquery("SELECT `categories_id` FROM `".TABLE_PRODUCTS_TO_CATEGORIES."` WHERE `categories_id` > 0 AND `products_id`='".$this->products_id."' LIMIT 0,1");
    $categorie_data=xtc_db_fetch_array($categorie_query, true);
    if (!empty($categorie_data)) {
      if ($categorie_data['categories_id']) {
        $this->product_data_raw['categories_id'] = $categorie_data['categories_id'];
      }
    }
    $this->product_data_use['categories_id'] = $this->product_data_raw['categories_id'];
  }
  
  private function check_special()
  {
    $this->product_data_use['product_will_have_special'] = 0;
    if (!empty($this->product_data_raw['special_start_date']) && substr($this->product_data_raw['special_start_date'],0,10) == '0000-00-00') {
      $this->product_data_raw['special_start_date'] = $this->product_data_use['special_start_date'] = '';
    }
    if (!empty($this->product_data_raw['special_expires_date']) && substr($this->product_data_raw['special_expires_date'],0,10) == '0000-00-00') {
      $this->product_data_raw['special_expires_date'] = $this->product_data_use['special_expires_date'] = '';
    }
    if ($this->product_data_raw['product_has_special'] && !empty($this->product_data_raw['special_start_date'])) {
      $tmp = strtotime($this->product_data_raw['special_start_date']);
      if ($tmp > time()) {
        $this->product_data_use['product_has_special'] = 0;
        $this->product_data_use['product_will_have_special'] = 1;
      }
    }
    if($this->product_data_raw['product_has_special'] && !empty($this->product_data_raw['special_expires_date'])) {
      $tmp = strtotime($this->product_data_raw['special_expires_date']);
      if ($tmp < time() && $tmp > 0) {
        $this->product_data_use['product_has_special'] = 0;
      }
      if ($this->product_data_raw['products_price'] > 0 && $this->product_data_raw['products_price'] < $this->product_data_raw['specials_new_products_price']) {
        $this->product_data_use['product_has_special'] = 0;
      }
    }
  }
  
  private function set_personal_price()
  {
    if (!$this->product_data_raw['product_has_special'] || $this->product_data_raw['products_price'] == 0) {
      if (AGI_SHOP_CONFIG::get_customers_permissions('customers_status_graduated_prices') && AGI_SHOP_CONFIG::get_agi_feed_conf('export_graduated_price')) {
        $offer_check = xtc_db_query("SELECT `personal_offer` FROM `".TABLE_PERSONAL_OFFERS_BY.AGI_SHOP_CONFIG::get_customers_group()."` WHERE `products_id`=".$this->products_id." AND `personal_offer`>0 ORDER BY `quantity` DESC LIMIT 0,1");
        if (xtc_db_num_rows($offer_check) == 1) {
          $tmp = xtc_db_fetch_array($offer_check);
          if (!empty($tmp['personal_offer'])) {
            $this->product_data_use['products_price'] = $tmp['personal_offer'];
            $this->product_data_raw['products_price'] = $tmp['personal_offer'];
          }
        }
      }
      else {
        $offer_check = xtc_db_query("SELECT `personal_offer` FROM `".TABLE_PERSONAL_OFFERS_BY.AGI_SHOP_CONFIG::get_customers_group()."` WHERE `products_id`=".$this->products_id." AND `personal_offer`>0 AND `quantity`=1");
        $tmp = xtc_db_fetch_array($offer_check);
        if (!empty($tmp)) {
          $this->product_data_use['products_price'] = $tmp['personal_offer'];
          $this->product_data_raw['products_price'] = $tmp['personal_offer'];
        }
      }
    }

    if (AGI_SHOP_CONFIG::get_add_price_percent() != 0) {
      $this->product_data_use['products_price'] = round($this->product_data_use['products_price']*(100+AGI_SHOP_CONFIG::get_add_price_percent())/100, 5);
      $this->product_data_raw['products_price'] = round($this->product_data_raw['products_price']*(100+AGI_SHOP_CONFIG::get_add_price_percent())/100, 5);
      $this->product_data_use['specials_new_products_price'] = round($this->product_data_use['specials_new_products_price']*(100+AGI_SHOP_CONFIG::get_add_price_percent())/100, 5);
    }
    
    if (AGI_SHOP_CONFIG::get_customers_permissions('customers_status_show_price_tax')) {
      $this->product_data_use['products_price'] = AGI_SHOP_CONFIG::add_tax(
        $this->product_data_use['products_price'], 
        AGI_SHOP_CONFIG::get_tax_rate($this->product_data_raw['products_tax_class_id'])
      );
      $this->product_data_raw['products_price'] = $this->product_data_use['products_price'];
      $this->product_data_use['specials_new_products_price'] = AGI_SHOP_CONFIG::add_tax(
        $this->product_data_use['specials_new_products_price'], 
        AGI_SHOP_CONFIG::get_tax_rate($this->product_data_raw['products_tax_class_id'])
      );
      $this->product_data_raw['specials_new_products_price'] = $this->product_data_use['specials_new_products_price'];
    }
    $products_discount = 0;
    if (!$this->product_data_raw['product_has_special']) {
      if (AGI_SHOP_CONFIG::get_customers_permissions('customers_status_discount') <> 0) {
        $products_discount = ($this->product_data_raw['products_discount_allowed'] < AGI_SHOP_CONFIG::get_customers_permissions('customers_status_discount')) ? $this->product_data_raw['products_discount_allowed'] : AGI_SHOP_CONFIG::get_customers_permissions('customers_status_discount');
      }
      if ($products_discount <> 0) {
        $this->product_data_use['products_price'] = $this->product_data_use['products_price']*(100-$products_discount)/100;
        $this->product_data_raw['products_price'] = $this->product_data_use['products_price'];
      }
    }
    $this->products_discount = $products_discount;
    return $this->product_data_use['products_price'];
  }
  
  private function create_link()
  {
    $pams = array();
    $prid = $this->products_id;
    if (!empty($this->my_attributes_options)) {
      $attribute = array();
      foreach ($this->my_attributes_options as $key => $value) {
        $attribute[] = '{'.$key.'}'.$value;
      }
      if (!empty($attribute)) {
        $prid .= implode('', $attribute);
      }
    }
    if (DEFAULT_LANGUAGE != AGI_SHOP_CONFIG::get_language_code()) {
      $pams[] = 'language='.AGI_SHOP_CONFIG::get_language_code();
    }
    $pams_string = !empty($pams) ? '&'.implode('&',$pams) : '';
    $this->product_data_use['link'] = xtc_href_link(
      FILENAME_PRODUCT_INFO, 
      xtc_product_link($prid).$pams_string,
      'NONSSL',
      false,
      empty($this->my_attributes_options)
    );
    $this->product_data_use['link'] = str_replace('&amp;','&',$this->product_data_use['link']);
    // Kampagnen
    $cmp = AGI_SHOP_CONFIG::get_campaign();
    if ($cmp != '') {
      $this->product_data_use['link'] .= strpos($this->product_data_use['link'], '?') ? '&'.$cmp : '?'.$cmp;
    }
    if (AGI_SHOP_CONFIG::is_currency_to_url()) {
      $this->product_data_use['link'] .= strpos($this->product_data_use['link'], '?') ? '&' : '?';
      $this->product_data_use['link'] .= 'currency='.AGI_SHOP_CONFIG::get_currency_value('CODE');
    }
    if (DEFAULT_LANGUAGE == AGI_SHOP_CONFIG::get_language_code() && isset($_GET['forcelang'])) {
      $this->product_data_use['link'] .= strpos($this->product_data_use['link'], '?') ? '&' : '?';
      $this->product_data_use['link'] .= 'language='.AGI_SHOP_CONFIG::get_language_code();
    }
    if (STORE_COUNTRY != AGI_SHOP_CONFIG::get_country_id()) {
      $this->product_data_use['link'] .= strpos($this->product_data_use['link'], '?') ? '&' : '?';
      $this->product_data_use['link'] .= 'agi_country='.AGI_SHOP_CONFIG::get_country_id();
    }
  }
  
  private function set_vpe()
  {
    if ($this->product_data_raw['products_vpe_status'] == 1 && $this->product_data_raw['products_vpe_value'] != 0.0 && $this->product_data_raw['products_price'] > 0) {
      if ($vpe_name = AGI_SHOP_CONFIG::get_vpe_data($this->product_data_raw['products_vpe']) && !AGI_SHOP_CONFIG::is_ignored_vpe_unit(AGI_SHOP_CONFIG::get_vpe_data($this->product_data_raw['products_vpe'], 'name'))) {
        $tmp_price = $this->product_data_use['product_has_special'] ? $this->product_data_use['specials_new_products_price'] : $this->product_data_use['products_price'];
        $vpe_price = AGI_SHOP_CONFIG::calculate_price($tmp_price * (1 / $this->product_data_raw['products_vpe_value']), 1).' pro '.AGI_SHOP_CONFIG::get_vpe_data($this->product_data_raw['products_vpe'], 'name');
        $this->product_data_use['vpe'] = $vpe_price;
        
        $tmp = number_format(AGI_SHOP_CONFIG::get_vpe_data($this->product_data_raw['products_vpe'], 'value')*$this->product_data_raw['products_vpe_value'],2,'.','');
        $this->product_data_use['unit_pricing_measure']      = $tmp.' '.AGI_SHOP_CONFIG::replace_vpe_unit(AGI_SHOP_CONFIG::get_vpe_data($this->product_data_raw['products_vpe'], 'unit'));
        $this->product_data_use['unit_pricing_base_measure'] = AGI_SHOP_CONFIG::get_vpe_data($this->product_data_raw['products_vpe'], 'value').' '.AGI_SHOP_CONFIG::replace_vpe_unit(AGI_SHOP_CONFIG::get_vpe_data($this->product_data_raw['products_vpe'], 'unit'));
      }
      else {
        $this->product_data_use['products_vpe_status'] = 0;
      }
    }
  }
  
  private function set_brand()
  {
    $this->product_data_use['brand'] = !empty($this->product_data_use['manufacturers_id']) ? AGI_SHOP_CONFIG::get_manufacturers_name($this->product_data_use['manufacturers_id']) : '';
    if (!empty($this->product_data_use['agi_export_brand'])) {
      $this->product_data_use['brand'] = $this->product_data_use['agi_export_brand'];
    }
  }
  
  private function set_category()
  {
    $pCategory = AGI_SHOP_CONFIG::get_categories_data($this->product_data_use['categories_id']);
    $this->product_data_use['category'] = $this->feed_type == 'google_xml' ? str_replace('>', '&gt;', $pCategory['path_html']) : $pCategory['path'];
    $use_google_category = AGI_SHOP_CONFIG::get_default_google_category();
    if (!empty($this->product_data_raw['agi_google_category'])) {
      $use_google_category = $this->product_data_raw['agi_google_category'];
    }
    elseif (!empty($pCategory['google'])) {
      $use_google_category = $pCategory['google'];
    }
    if (preg_match('/^(\d+) -/', $use_google_category, $matches)) {
      $this->product_data_use['agi_google_category'] = $matches[1];
    }
    else {
      $this->product_data_use['agi_google_category'] = $this->feed_type == 'google_xml' ? str_replace('>', '&gt;', $use_google_category) : $use_google_category;
    }
  }
  
  private function set_images()
  {
    if (!empty($this->product_data_raw['products_image'])) {
      $this->product_images[0] = $this->product_data_raw['products_image'];
    }
    $images = xtc_get_products_mo_images($this->products_id);
    if ($images) {
      foreach ($images as $image) {
        $this->product_images[$image['image_nr']] = $image['image_name'];
      }
    }
    $this->product_data_use['products_images'] = $this->product_images;
  }
  
  private function get_attributes_images($options_id, $options_values_id)
  {
    if (empty($this->product_attribute_images)) {
      return array();
    }
    $found_images = array();
    foreach ($this->product_attribute_images as $key => $row) {
      foreach ($row as $attributes) {
        if ($attributes['o'] == $options_id && ($attributes['v'] == $options_values_id || $attributes['v'] == 0)) {
          $found_images[] = $key;
        }
      }
    }
    return array_unique($found_images);
  }
  
  private function load_attribute_images()
  {
    $this->product_attribute_images = array();
    if (!empty($this->product_data_raw['agi_export_image_json'])) {
      $tmp = json_decode($this->product_data_raw['agi_export_image_json'], true);
      foreach ($tmp as $img_nr => $img_data) {
        if (!empty($img_data['attributes'])) {
          $this->product_attribute_images[$img_nr] = $img_data['attributes'];
        }
      }
    }
  }
  
  private function set_availability($preset = '', $q = null)
  {
    $availability = empty($preset) ? 'in stock' : $preset;
    $quantity = $q !== null ? $q : $this->product_data_use['products_quantity'];
    
    if (AGI_SHOP_CONFIG::get_agi_export_conf('availability_base') == 'QUANTITY') {
      if ($quantity <= 0) {
        $availability = 'out of stock';
      }
      else {
        $availability = 'in stock';
      }
    }
    elseif (AGI_SHOP_CONFIG::get_agi_export_conf('availability_base') == 'SHIPPING_TIME') {
      $availability = AGI_SHOP_CONFIG::get_google_shipping_status($this->product_data_use['products_shippingtime']);
    }
    $this->product_data_use['google_availability_status'] = empty($availability) ? 'in stock' : $availability;
    return $this->product_data_use['google_availability_status'];
  }
  
  private function prepare_attributes()
  {
    // get attributes set in product-details
    $this->my_google_attribute_options_ids = AGI_SHOP_CONFIG::get_google_attribute_options_ids();
    $this->my_google_attribute_options_ids_name = AGI_SHOP_CONFIG::get_google_attribute_options_ids_name();
    $allowed_attributes = array('color','size','pattern','material');
    $this->overwrite_attributes = array();
    $this->max_overwrite_attributes = 1;
    foreach ($allowed_attributes as $a_name) {
      $tmp = preg_replace('/\s/', '', $this->product_data_use['agi_attribute_'.$a_name]);
      if (!empty($tmp)) {
        $this->overwrite_attributes[$a_name] = explode(';', $this->product_data_use['agi_attribute_'.$a_name]);
        $this->max_overwrite_attributes = count($this->overwrite_attributes[$a_name]) > $this->max_overwrite_attributes ? count($this->overwrite_attributes[$a_name]) : $this->max_overwrite_attributes;
        if (!array_key_exists($a_name, $this->my_google_attribute_options_ids_name)) {
          $this->my_google_attribute_options_ids_name[$a_name] = array();
        }
        AGI_SHOP_CONFIG::set_google_attribute_options($a_name);
        if (!empty($this->my_google_attribute_options_ids_name[$a_name])) {
          $this->my_google_attribute_options_ids = array_diff($this->my_google_attribute_options_ids, $this->my_google_attribute_options_ids_name[$a_name]);
        }
      }
    }
  }
  
  private function attribute_handling()
  {
    if (!empty($this->my_google_attribute_options_ids) || !empty($this->overwrite_attributes)) {
      $this->my_attributes_options = array(); // AGI Attribut-Link
      $allready_print_out = false;
      $attributes = array(); // entries of table 'products_attributes'
      $attributes_to_options = array();
      $found_db_attributes = 0;
      $_addup_vpes = AGI_SHOP_CONFIG::get_agi_export_conf('add_attributes_vpe');
      // get options from database
      if (!empty($this->my_google_attribute_options_ids)) {
        // check for attributes
        $a_check = "SELECT COUNT(*) AS total 
        FROM 
          `".TABLE_PRODUCTS_ATTRIBUTES."` 
        WHERE 
          `products_id` = ".$this->products_id." AND 
          `options_id` IN (".implode(',', $this->my_google_attribute_options_ids).")
        ";
        $a_check = xtc_db_query($a_check);
        $a_check_result = xtc_db_fetch_array($a_check);
        $found_db_attributes = $a_check_result['total'];
        if ($found_db_attributes > 0) {
          // we've got attributes
          // get all attributes
          $attributes_query = xtc_db_query("SELECT pa.*, pav.products_options_values_name FROM `".TABLE_PRODUCTS_ATTRIBUTES."` AS pa, `".TABLE_PRODUCTS_OPTIONS_VALUES."` AS pav WHERE pa.products_id=".$this->products_id." AND pa.options_id IN (".implode(',', $this->my_google_attribute_options_ids).") AND pav.products_options_values_id=pa.options_values_id AND pav.language_id=".AGI_SHOP_CONFIG::get_language_id()." ORDER BY `options_id`, `sortorder`");
          while ($db_row = xtc_db_fetch_array($attributes_query)) {
            if (!array_key_exists($db_row['options_id'], $attributes_to_options)) {
              $attributes_to_options[$db_row['options_id']] = array();
            }
            $attributes_to_options[$db_row['options_id']][] = $db_row['products_attributes_id'];
            $attributes[$db_row['products_attributes_id']] = $db_row;
          }
        }
      }
      
      // get manually set options
      if (!empty($this->overwrite_attributes)) {
        foreach ($this->overwrite_attributes as $google_name => $overrite_values) {
          if (!array_key_exists($google_name, $attributes_to_options)) {
            $attributes_to_options[$google_name] = array();
          }
          foreach ($overrite_values as $key => $value) {
            $attributes_to_options[$google_name][] = $google_name.'_'.$key;
            $attributes[$google_name.'_'.$key] = array(
              'products_attributes_id'    => $google_name.'_'.$key,
              'products_id'               => $this->products_id,
              'options_id'                => $google_name,
              'options_values_id'         => 0,
              'options_values_price'      => 0,
              'price_prefix'              => '+',
              'attributes_model'          => '',
              'attributes_stock'          => $this->product_data_use['products_quantity'],
              'options_values_weight'     => 0,
              'weight_prefix'             => '+',
              'sortorder'                 => 0,
              'products_options_values_name' => trim($value),
              'attributes_vpe_value'      => 0,
              'attributes_vpe_id'         => 0,
              'attributes_ean'            => ''
            );
          }
        }
      }
      if ($found_db_attributes > 0 || $this->max_overwrite_attributes > 1) {
        $cnt_attributes = 0;
        $this->load_attribute_images(); // Attributbilder
        $attribute_response = $this->combine_attributes($attributes_to_options); // combination of all attributes
        $overwrite_products_data = array();
        foreach ($attribute_response as $attribut_combination) {
          $var_amount = 0;
          $tmp_attribute_vpe_value = 0;
          $tmp_attribute_vpe_value_id = 0;
          $var_price  = round($this->product_data_raw['products_price'],2);
          $pdif = 0;
          $var_reduced_price = round($this->product_data_use['specials_new_products_price'],2);
          $var_weight = $this->product_data_use['products_weight'];
          $var_vpe_value = $this->product_data_raw['products_vpe_value'];
          $this->product_data_use['item_group_id'] = $this->products_id;
          $tmp_pid = $this->products_id;
          $tmp_google_attributes = array();
          $tmp_attributes_images = array();
          foreach ($this->my_google_attribute_options_ids_name as $key => $value) {
            $tmp_google_attributes[$key] = '';
          }

          $i_count = 0;
          foreach ($attribut_combination as $attribut_key) {
            $attribute_data = $attributes[$attribut_key];
            if ($attribute_data['options_values_id'] > 0) {
              $this->my_attributes_options[$attribute_data['options_id']] = $attribute_data['options_values_id'];
            }
            if (!empty($attribute_data['products_options_values_name'])) {
              $tmp_google_attributes[AGI_SHOP_CONFIG::get_google_attribute_option($attribute_data['options_id'])] .= empty($tmp_google_attributes[AGI_SHOP_CONFIG::get_google_attribute_option($attribute_data['options_id'])]) ? $attribute_data['products_options_values_name'] : '-'.$attribute_data['products_options_values_name'];
            }
            $tmp_pid .= empty($attribute_data['options_values_id']) ? '-'.$attribute_data['products_attributes_id'] : '{'.$attribute_data['options_id'].'}'.$attribute_data['options_values_id'];
            
            // get amount
            if ($i_count==0) {
              $var_amount = $attribute_data['attributes_stock'];
            }
            else {
              $var_amount = ($var_amount>$attribute_data['attributes_stock']) ? $attribute_data['attributes_stock'] : $var_amount;
            }
            
            // calculate variant-price
            if (!empty($attribute_data['options_values_price'])) {
              $values_price = (AGI_SHOP_CONFIG::get_customers_permissions('customers_status_show_price_tax')) ? AGI_SHOP_CONFIG::add_tax($attribute_data['options_values_price'], AGI_SHOP_CONFIG::get_tax_rate($this->product_data_raw['products_tax_class_id'])) : $attribute_data['options_values_price'];
              if (AGI_SHOP_CONFIG::get_add_price_percent() != 0) {
                $values_price = round($values_price*(100+AGI_SHOP_CONFIG::get_add_price_percent())/100, 5);
              }
              switch($attribute_data['price_prefix']) {
                case '-': 
                    $pdif += $values_price;
                    //$var_price += $values_price;
                    //$var_reduced_price += $values_price;
                  break;
                case '=': 
                    //$pdif += $values_price;
                    $var_price = $values_price;
                    $var_reduced_price = $values_price;
                  break;
                default: 
                    $pdif += $values_price;
                    //$var_price += $values_price;
                    //$var_reduced_price += $values_price;
              }
            }
            if ($this->products_discount <> 0 && AGI_SHOP_CONFIG::get_customers_permissions('customers_status_discount_attributes')) {
              $overwrite_products_data['products_price'] = $var_price+($pdif*(100-$this->products_discount)/100);
            }
            else {
              $overwrite_products_data['products_price'] = $var_price+$pdif;
            }
            $overwrite_products_data['specials_new_products_price'] = $var_reduced_price+$pdif;
            // calculate variants-weight
            if (!empty($attribute_data['options_values_weight'])) {
              switch($attribute_data['weight_prefix']) {
                case '-': $var_weight -= $attribute_data['options_values_weight']; break;
                case '=': $var_weight = $attribute_data['options_values_weight'];break;
                default: $var_weight += $attribute_data['options_values_weight'];
              }
              $overwrite_products_data['products_weight'] = $var_weight;
            }
            
            if (!empty($attribute_data['attributes_ean'])) {
              $overwrite_products_data['products_ean'] = $attribute_data['attributes_ean'];
            }
              
            if ($attribute_data['attributes_vpe_value'] <> 0) {
              if ($_addup_vpes) {
                switch($attribute_data['weight_prefix']) {
                  case '-': $var_vpe_value -= $attribute_data['attributes_vpe_value']; break;
                  case '=': $var_vpe_value = $attribute_data['attributes_vpe_value']; break;
                  default: $var_vpe_value += $attribute_data['attributes_vpe_value'];
                }
              } else {
                if ($attribute_data['attributes_vpe_value'] > $var_vpe_value) {
                  $var_vpe_value = $attribute_data['attributes_vpe_value'];
                }
              }
            }

            $tmp = $this->get_attributes_images($attribute_data['options_id'], $attribute_data['options_values_id']);
            foreach ($tmp as $img_nr) {
              if (!array_key_exists($img_nr, $tmp_attributes_images)) {
                $tmp_attributes_images[$img_nr] = 0;
              }
              $tmp_attributes_images[$img_nr]++;
            }
            
            $i_count++;
          }
          $overwrite_products_data['products_id'] = $tmp_pid;
          
          foreach ($this->my_google_attribute_options_ids_name as $key => $value) {
            if (!empty($tmp_google_attributes[$key])) {
              $overwrite_products_data[empty($this->product_data_use['agi_attribute_'.$key]) ? 'pvariant_attribute_'.$key : 'agi_attribute_'.$key] = $tmp_google_attributes[$key];
            }
          }
          
          // Verfügbarkeit neu berechnen
          $quantity = $var_amount;
          if (count($attribut_combination) == 1 && empty($attribute_data['attributes_ean'])) {
            $quantity = $this->product_data_use['products_quantity'];
          }
          if ($quantity < 0) {
            $quantity = 0;
          }
          $overwrite_products_data['products_quantity'] = $quantity;
          $overwrite_products_data['google_availability_status'] = $this->set_availability($this->product_data_use['google_availability_status'], $overwrite_products_data['products_quantity']);
          
          // VPE neu berechnen
          if ($this->product_data_use['products_vpe_status'] == 1 && $var_vpe_value != 0.0 && $this->product_data_raw['products_price'] > 0) {
            $use_vpe_id = $tmp_attribute_vpe_value_id == 0 ? $this->product_data_raw['products_vpe'] : $tmp_attribute_vpe_value_id;
            if ($vpe_name = AGI_SHOP_CONFIG::get_vpe_data($use_vpe_id) && !AGI_SHOP_CONFIG::is_ignored_vpe_unit(AGI_SHOP_CONFIG::get_vpe_data($use_vpe_id, 'name'))) {
              $tmp_price = $this->product_data_use['product_has_special'] ? $overwrite_products_data['specials_new_products_price'] : $overwrite_products_data['products_price'];
              $vpe_price = AGI_SHOP_CONFIG::calculate_price($tmp_price * (1 / $var_vpe_value), 1).' pro '.AGI_SHOP_CONFIG::get_vpe_data($use_vpe_id, 'name');
              $overwrite_products_data['vpe'] = $vpe_price;
              
              $tmp = number_format(AGI_SHOP_CONFIG::get_vpe_data($use_vpe_id, 'value')*$var_vpe_value,2,'.','');
              $overwrite_products_data['unit_pricing_measure']      = $tmp.' '.AGI_SHOP_CONFIG::replace_vpe_unit(AGI_SHOP_CONFIG::get_vpe_data($use_vpe_id, 'unit'));
              $overwrite_products_data['unit_pricing_base_measure'] = AGI_SHOP_CONFIG::get_vpe_data($use_vpe_id, 'value').' '.AGI_SHOP_CONFIG::replace_vpe_unit(AGI_SHOP_CONFIG::get_vpe_data($use_vpe_id, 'unit'));
              $overwrite_products_data['products_vpe'] = $use_vpe_id;
              $overwrite_products_data['products_vpe_value'] = $var_vpe_value;
            }
            else {
              $overwrite_products_data['products_vpe_status'] = 0;
            }
          }
          
          // Versandgewicht anpassen
          $this->product_data_use['products_price'] = $overwrite_products_data['products_price'];
          $this->calculate_shipping();
          $this->calculate_morecountry_shipping();
          if (!AGI_EXPORT_QUICK) {
            $this->set_products_texts();
            // Attributbilder setzen
            if (!empty($tmp_attributes_images)) {
              arsort($tmp_attributes_images);
              $overwrite_products_data['products_images'] = array();
              foreach ($tmp_attributes_images as $img_nr => $i_sort) {
                $overwrite_products_data['products_images'][$img_nr] = $this->product_images[$img_nr];
              }
            }
            else {
              unset($overwrite_products_data['products_images']);
            }
            if (!empty($this->my_attributes_options)) {
              $this->create_link();
            }
          }
          
          // Produkt hier ausgeben
          $cache_imgs = $this->product_images;
          $this->write_item_data($overwrite_products_data);
          $this->product_images = $cache_imgs;
          $allready_print_out = true;
          $cnt_attributes++;
          if (AGI_SHOP_CONFIG::is_attribut_limit() && $cnt_attributes>AGI_SHOP_CONFIG::is_attribut_limit()) {
            return true;
          }
        }
      }
      return $allready_print_out;
    }
    return false;
  }
  
  private function set_products_texts()
  {
    $products_description = AGI_SHOP_CONFIG::get_agi_export_conf('use_description') == 'short' ? $this->product_data_raw['products_short_description'] : $this->product_data_raw['products_description'];
    if (strpos($products_description, '[TAB:') !== false) {
      $products_description = preg_replace('/\[TAB:(.+?)\]/', '<br /><br />$1<br />', $products_description);
    }
    $products_description = $this->clean_html_to_text($products_description);
    $products_name = empty($this->product_data_raw['agi_alternate_title']) ? $this->product_data_raw['products_name'] : $this->product_data_raw['agi_alternate_title'];
    $this->product_data_use['products_description'] = $products_description;
    $this->product_data_use['products_name'] = $products_name;
    
    $agi_highlight_text = array();
    if (!empty($this->product_data_raw['agi_highlight_text'])) {
      $substr = substr($this->product_data_raw['agi_highlight_text'], 0, 1);
      if (in_array($substr, array('{','['))) {
        $tmp = json_decode($this->product_data_raw['agi_highlight_text'], true);
        foreach ($tmp as $value) {
          $agi_highlight_text[] = decode_utf8($value);
        }
      } else {
        $agi_highlight_text = array(decode_utf8($this->product_data_raw['agi_highlight_text']));
      }
    }
    $this->product_data_use['agi_highlight_text'] = $agi_highlight_text;
  }
  
  public function write_item_data($overwrite_products_data = array())
  {
    $products_data = $this->product_data_use;
    foreach($overwrite_products_data as $key => $value) {
      if (!array_key_exists($key, $products_data)) {
        $products_data[$key] = $value;
      }
      elseif (!empty($value)) {
        $products_data[$key] = $value;
      }
    }
    
    $this->build_product_item($products_data, $overwrite_products_data);
    $this->print_out_product();
  }
}
