<?php
/* includes/external/agi/agi_export/export/classes/xml_feed_builder.class.php
.---------------------------------------------------------------------------.
|    Software: agi_export for modified eCommerce                            |
|      Author: Andreas Guder                                                |
|     Contact: info@andreas-guder.de                                        |
| Copyright (c) 2022, Andreas Guder [info@andreas-guder.de]                 |
|               All Rights Reserved.                                        |
| ------------------------------------------------------------------------- |
|     License: you are not allowed to distribute                            |
|              or sublicense copies of the software without                 |
|              a grant of a licence of the coyright-owner                   |
'---------------------------------------------------------------------------ü
*/

class XML_FEED_BUILDER
{
  private $output_fp = null;
  private $tmpXMLString = array();
  private $tmpDataArray = array();
  private $item_data = array();
  
  private $filename_base = '';
  protected $feed_type = 'google_xml';
  private $feed_charset = 'UTF-8';
  private $shop_utf8 = false;
  private $recode_value = false;

  public function __construct($filename, $settings) //$shop_utf8, $feed_charset)
  {
    $this->filename_base = $filename;
    if (array_key_exists('charset', $settings)) {
      $this->feed_charset = strtoupper($settings['charset']);
    }
    if (array_key_exists('is_utf8_shop', $settings)) {
      $this->shop_utf8 = $settings['is_utf8_shop'];
    }
    $this->recode_value = (($this->shop_utf8 && $this->feed_charset == 'UTF-8') || (!$this->shop_utf8 && $this->feed_charset != 'UTF-8')) ? false : true;
  }
  
  public function start_feed()
  {
    $this->output_fp = fopen($this->filename_base.'.tmp', 'a');
    $this->tmpXMLString = '';
  }
  public function end_feed()
  {
    fclose($this->output_fp);
  }
  public function write_feed_line($tring)
  {
    fwrite($this->output_fp, $tring.chr(10));
  }
  public function add_feed_item()
  {
    $string = $this->create_xml_from_array($this->item_data);
    $this->write_feed_line($string);
  }
  public function charset_coding_string($value)
  {
    if (!$this->recode_value) {
      return $value;
    }
    return $this->shop_utf8 ? mb_convert_encoding($value, $this->feed_charset, 'UTF-8') : mb_convert_encoding($value, 'UTF-8', $_SESSION['language_charset']);
  }
  
  private function create_xml_from_array($aArray)
  {
    $this->tmpXMLString = '';
    if (!array_key_exists('value', $aArray) || empty($aArray['value'])) {
      $aArray['value'] = '';
    }
    $this->tmpXMLString .= $this->get_xml_tag(
      array_key_exists('name', $aArray) && !empty($aArray['name']) ? $aArray['name'] : 'item', 
      is_array($aArray['value']) ? '' : $aArray['value'], 
      is_array($aArray['value']) ? true : false, 
      array_key_exists('attribute', $aArray) && !empty($aArray['attribute']) ? $aArray['attribute'] : array()
    );
    
    if (is_array($aArray['value']) && !empty($aArray['value'])) {
      $this->tmpXMLString_addChildren($aArray['value']);
      $this->tmpXMLString .= $this->close_xml_tag(array_key_exists('name', $aArray) && !empty($aArray['name']) ? $aArray['name'] : 'item');
    }
    return $this->tmpXMLString;
  }
  private function tmpXMLString_addChildren($aArray)
  {
    foreach ($aArray as $key => $value) {
      if (is_array($value)) {
        if (!array_key_exists('value', $value) || empty($value['value'])) {
          $value['value'] = '';
        }
        if (is_array($value['value']) && !empty($value['value']) && is_numeric($value['value'][0]['name'])) {
          foreach ($value['value'] as $row) {
            if (is_array($row['value'])) {
              $this->tmpXMLString .= $this->get_xml_tag($key, '', true);
              $this->tmpXMLString_addChildren($row['value']);
              $this->tmpXMLString .= $this->close_xml_tag($key);
            }
            else {
              $this->tmpXMLString .= $this->get_xml_tag(
                array_key_exists('name', $value) && !empty($value['name']) ? $value['name'] : $key, 
                is_array($row['value']) ? '' : $row['value'], 
                is_array($row['value']) ? true : false, 
                array_key_exists('attribute', $row) && !empty($value['attribute']) ? $row['attribute'] : array()
              );
            }
          }
        }
        else {
          $this->tmpXMLString .= $this->get_xml_tag(
            array_key_exists('name', $value) && !empty($value['name']) ? $value['name'] : $key, 
            is_array($value['value']) ? '' : $value['value'], 
            is_array($value['value']) ? true : false, 
            array_key_exists('attribute', $value) && !empty($value['attribute']) ? $value['attribute'] : array()
          );
          if (is_array($value['value']) && !empty($value['value'])) {
            $this->tmpXMLString_addChildren($value['value']);
            $this->tmpXMLString .= $this->close_xml_tag(array_key_exists('name', $value) && !empty($value['name']) ? $value['name'] : $key);
          }
        }
      }
      else {
        $this->tmpXMLString .= $this->get_xml_tag($key, $value);
      }
    }
  }
  /**
   * return attributes string of xml-tag
   * @param array of attributes ($attribute => $value,...)
   * @return string
   */
  private function get_tag_attributes_string($attribute)
  {
    $string = '';
    foreach ($attribute as $key => $value) {
      $string .= ' ';
      $string .= $key.'='.$this->quote_attribute($value, true);
    }
    return $string;
  }
  /**
   * return closing-tag
   * @param string
   * @return string
   */
  private function close_xml_tag($tag)
  {
    return '</'.$tag.'>'.chr(10);
  }
  /**
   * return xml-tag
   * @param string $tag
   * @param string $value
   * @param bool tag has childs
   * @param array list of attributes
   * @param $maxlength
   * @return string
   */
  private function get_xml_tag($tag, $value = '', $childs = false, $attributes = array(), $maxlength = null)
  {
    $closed = false;
    $string = '<'.$tag;
    if (!empty($attributes)) {
      $string .= $this->get_tag_attributes_string($attributes);
    }
    if (is_string($value) && $value == '' && !$childs) {
      $string .= ' />'.chr(10);
      $closed = true;
    }
    else {
      $string .= '>';
    }

    if (is_string($value) && (strtolower($value) != 'true' && strtolower($value) != 'false')) {
      if ($value != '') {
        if (strpos($value, '<') !== false || strpos($value, '>') !== false || strpos($value, '&') !== false) {
          $string .= '<![CDATA['.$this->charset_coding_string($maxlength ? $this->short_string($value, $maxlength) : $value).']]>';
        } else {
          $string .= $this->charset_coding_string($maxlength ? $this->short_string($value, $maxlength) : $value);
        }
      }
      else {
        $string .= $value;
      }
    }
    else {
      $string .= $value;
    }
    
    if (!$childs && !$closed) {
      $string .= $this->close_xml_tag($tag);
    } elseif ($childs && !$closed) {
      $string .= chr(10);
    }
    
    return $string;
  }
  /**
   * @param mixed
   */
  private function quote_attribute($value, $quote_always=false)
  {
    if ($quote_always) {
      return '"'.$this->charset_coding_string($value).'"';
    }
    return is_string($value) ? '"'.$this->charset_coding_string($value).'"' : $this->charset_coding_string($value);
  }
  
  public function short_string($string, $maxlength)
  {
    if (mb_strlen($string) <= $maxlength) {
      return $string;
    }
    $shorten = wordwrap($string, $maxlength, "||");
    $shorten = explode("||", $shorten);
    return $shorten[0];
  }
  
  public function add_item_data($key, $value, $attribute = array())
  {
    if (is_array($value)) {
      $this->item_data['value'][$key] = array('value' => array());
      foreach ($value as $akey => $row) {
        $this->item_data['value'][$key]['value'][] = array(
          'name' => $akey,
          'value'=> $row,
          'attribute' => empty($attribute) ? array() : $attribute
        );
      }
    }
    else {
      $this->item_data['value'][$key] = $value;
    }
  }
  
  public function clear_item_data()
  {
    $this->item_data = array(
      'name' => 'item',
      'value' => array()
    );
  }
  
  protected function build_product_item($products_data, $overwritten_products_data) {
    if (!AGI_EXPORT_QUICK) {
      if (!empty($products_data['vpe'])) {
        if (AGI_SHOP_CONFIG::get_agi_export_conf('vpe_to_title')) {
          $products_data['products_name'] .= ' ('.$products_data['vpe'].')';
        }
        if (AGI_SHOP_CONFIG::get_agi_export_conf('vpe_to_desc')) {
          $products_data['products_description'] = '('.$products_data['vpe'].') '.$products_data['products_description'];
        }
      }
    
      $this->add_item_data('title', $this->short_string($products_data['products_name'], 150));
      $this->add_item_data('description', $this->short_string($products_data['products_description'], 5000));
      $this->add_item_data('link', $products_data['link']);
    }

    $this->add_item_data('g:id', $this->short_string($products_data['products_id'], 50));
    if (!empty($products_data['agi_export_pgroup_id'])) {
      $this->add_item_data('g:item_group_id', $this->short_string($products_data['agi_export_pgroup_id'], 50));
    }
    elseif (!empty($products_data['item_group_id'])) {
      $this->add_item_data('g:item_group_id', $this->short_string($products_data['item_group_id'], 50));
    }
    if (!AGI_EXPORT_QUICK) {
      $this->add_item_data('g:google_product_category', $products_data['agi_google_category']);
      $this->add_item_data('g:product_type', $this->short_string($products_data['category'], 750));
      $image_dir = DIR_WS_POPUP_IMAGES;
      $use_image = AGI_SHOP_CONFIG::get_agi_export_conf('products_image');
      if ($use_image == 'info_images') {
        $image_dir = DIR_WS_INFO_IMAGES;
      } elseif ($use_image == 'original_images') {
        $image_dir = DIR_WS_ORIGINAL_IMAGES;
      }
      
      $export_images = array();
      $allowed_images = array();
      if (!empty($products_data['agi_export_image_json'])) {
        $tmp = json_decode($products_data['agi_export_image_json'], true);
        $max_sort = 0;
        foreach ($tmp as $img_nr => $img_data) {
          if ($img_data['export']) {
            $allowed_images[$img_nr] = empty($img_data['sortorder']) ? 0 : $img_data['sortorder'];
            if ($img_data['sortorder'] !== "") {
              $max_sort = $img_data['sortorder'] > $max_sort ? $img_data['sortorder'] : $max_sort;
            }
          }
        }
        foreach ($allowed_images as $img_nr => $i_sort) {
          if ($i_sort === "") {
            $max_sort++;
            $allowed_images[$img_nr] = $max_sort;
          }
        }
        if (!array_key_exists('products_images', $overwritten_products_data)) {
          asort($allowed_images);
          foreach ($allowed_images as $img_nr => $i_sort) {
            if (array_key_exists($img_nr, $products_data['products_images'])) {
              $export_images[] = $products_data['products_images'][$img_nr];
            }
          }
          $products_data['products_images'] = $export_images;
        } else {
          foreach ($products_data['products_images'] as $img_nr => $img_path) {
            if (array_key_exists($img_nr, $allowed_images)) {
              $export_images[] = $img_path;
            }
          }
          $products_data['products_images'] = $export_images;
        }
      }
      $i_count = 0;
      $additional_images = array();
      foreach ($products_data['products_images'] as $row) {
        if (defined('IMAGE_TYPE_EXTENSION') && IMAGE_TYPE_EXTENSION != 'default') {
          $image_extension = substr($row, 0, strrpos($row, '.')).'.'.IMAGE_TYPE_EXTENSION;
          if (is_file(DIR_FS_CATALOG . $image_dir . $image_extension)) {
            $row = $image_extension;
          }
        }
        if ($i_count == 0) {
          $this->add_item_data('g:image_link', HTTP_SERVER . DIR_WS_CATALOG . $image_dir . $row);
        } else {
          $additional_images[] = HTTP_SERVER . DIR_WS_CATALOG . $image_dir . $row;
        }
        $i_count++;
        if ($i_count >= 10) {
          break;
        }
      }
      if (!empty($additional_images)) {
        $this->add_item_data('g:additional_image_link', $additional_images);
      }
    }
    $this->add_item_data('g:condition', $products_data['agi_export_condition']);
    
    
    if (!empty($products_data['products_date_available']) && $products_data['products_date_available'] > date('Y-m-d H:i:s')) {
      $this->add_item_data('g:availability', 'preorder');
      $tmp = strtotime($products_data['special_expires_date'].' +1 day');
      $this->add_item_data('g:availability_date', date('Y-m-d',$tmp).'T'.date('H:iO',$tmp));
    } else {
      $this->add_item_data('g:availability', $products_data['google_availability_status']);
      if (in_array($products_data['google_availability_status'], array('preorder','backorder'))) {
        $s_status = strtolower(AGI_SHOP_CONFIG::get_shipping_status_name($products_data['products_shippingtime']));
        if (preg_match('/\d+/', $s_status, $matches)) {
          if (strpos($s_status, 'day') || strpos($s_status, 'tag') || strpos($s_status, 'jour')) {
            $tmp = strtotime('+'.$matches[0].' day');
            $this->add_item_data('g:availability_date', date('Y-m-d',$tmp).'T'.date('H:iO',$tmp));
          } elseif (strpos($s_status, 'week') || strpos($s_status, 'woche') || strpos($s_status, 'semaine')) {
            $tmp = strtotime('+'.$matches[0].' week');
            $this->add_item_data('g:availability_date', date('Y-m-d',$tmp).'T'.date('H:iO',$tmp));
          } elseif (strpos($s_status, 'month') || strpos($s_status, 'monat') || strpos($s_status, 'mois')) {
            $tmp = strtotime('+'.$matches[0].' month');
            $this->add_item_data('g:availability_date', date('Y-m-d',$tmp).'T'.date('H:iO',$tmp));
          } elseif (strpos($s_status, 'year') || strpos($s_status, 'jahr') || strpos($s_status, 'an')) {
            $tmp = strtotime('+'.$matches[0].' day');
            $this->add_item_data('g:availability_date', date('Y-m-d',$tmp).'T'.date('H:iO',$tmp));
          }
        }
      }
    }
    $this->add_item_data('g:price', AGI_SHOP_CONFIG::calculate_price($products_data['products_price'], 2));
    if ($products_data['product_has_special'] || $products_data['product_will_have_special']) {
      $special_start = !empty($products_data['special_start_date']) ? strtotime($products_data['special_start_date']) : time();
      $this->add_item_data('g:sale_price', AGI_SHOP_CONFIG::calculate_price($products_data['specials_new_products_price'], 2));
      if (!empty($products_data['special_start_date']) || !empty($products_data['special_expires_date'])) {
        if (!empty($products_data['special_expires_date'])) {
          $tmp = strtotime($products_data['special_expires_date'].' +1 day');
          if ($tmp > 0) {
            $this->add_item_data('g:sale_price_effective_date', date('Y-m-d',$special_start).'T00:00'.date('O').'/'.date('Y-m-d',$tmp).'T'.date('H:iO',$tmp));
          }
        } else {
          $tmp = time()+10*86400;
          $this->add_item_data('g:sale_price_effective_date', date('Y-m-d',$special_start).'T00:00'.date('O').'/'.date('Y-m-d',$tmp).'T'.date('H:iO',$tmp));
        }
      }
    }

    if (!AGI_EXPORT_QUICK) {
      $ident_count = 0;
      if (!empty($products_data['brand'])) {
        $this->add_item_data('g:brand', $this->short_string($products_data['brand'], 70));
        $ident_count++;
      }
      if (!empty($products_data['products_ean']) && strlen($products_data['products_ean']) > 5) {
        $this->add_item_data('g:gtin', $this->short_string($products_data['products_ean'], 50));
        $ident_count++;
      }
      $mpn = !empty($products_data['products_manufacturers_model']) ? $products_data['products_manufacturers_model'] : $products_data['products_model'];
      if (!empty($mpn)) {
        $this->add_item_data('g:mpn', $this->short_string($mpn, 70));
        $ident_count++;
      }
      if ($products_data['agi_identifier_exists'] == 1 && $ident_count < 2) {
        $this->add_item_data('g:identifier_exists', 'FALSE');
        $this->add_item_data('g:mpn', '');
      }
      else {
        $this->add_item_data('g:identifier_exists', $products_data['agi_identifier_exists'] ? 'TRUE' : 'FALSE');
        if (!$products_data['agi_identifier_exists']) {
          $this->add_item_data('g:mpn', '');
        }
      }
    }
    
    $_gsetting = AGI_SHOP_CONFIG::get_agi_feed_conf('google_settings');
    if (empty($_gsetting) || (is_array($_gsetting) && array_key_exists('do_shipping', $_gsetting) && $_gsetting['do_shipping'])) {
      $list = AGI_SHOP_CONFIG::get_country_shipping_list();
      if (!empty($list)) {
        $tmp = array();
        $shipping_array = array(
          'g:country' => AGI_SHOP_CONFIG::get_country_code(),
          'g:service' => $products_data['shipping_price'] > 0 ? MODULE_ORDER_TOTAL_SHIPPING_TITLE : FREE_SHIPPING_TITLE,
          'g:price'   => AGI_SHOP_CONFIG::calculate_price($products_data['shipping_price'], 2)
        );
        $tmp[] = $shipping_array;
        foreach($list as $country) {
          $tprice = array_key_exists('shipping_price_'.$country, $products_data) ? $products_data['shipping_price_'.$country] : $products_data['shipping_price'];
          $shipping_array = array(
            'g:country' => $country,
            'g:service' => $tprice > 0 ? MODULE_ORDER_TOTAL_SHIPPING_TITLE : FREE_SHIPPING_TITLE,
            'g:price'   => AGI_SHOP_CONFIG::calculate_price($tprice, 2)
          );
          $tmp[] = $shipping_array;
        }
        $this->add_item_data('g:shipping', $tmp);
      }
      else {
        $shipping_array = array(
          'g:country' => AGI_SHOP_CONFIG::get_country_code(),
          'g:service' => $products_data['shipping_price'] > 0 ? MODULE_ORDER_TOTAL_SHIPPING_TITLE : FREE_SHIPPING_TITLE,
          'g:price'   => AGI_SHOP_CONFIG::calculate_price($products_data['shipping_price'], 2)
        );
        $this->add_item_data('g:shipping', $shipping_array);
      }
    } else {
      $this->add_item_data('g:shipping_label', $products_data['shipping_price'] > 0 ? 'standardversand' : 'kostenfrei');
    }
    
    if (!in_array(AGI_SHOP_CONFIG::get_country_code(), array('DE','AT','CH','AR','AU','BE','BR','CL','DK','FR','HK','IN','ID','IE','IL','IT','JP','MY','MX','NZ','NL','NO','PH','PL','PT','RU','SA','SE','SG','ES','ZA','KR','TW','TH','TR','CZ','UA','AE','UK','VN'))) {
      $tax_array = array(
        'g:rate' => AGI_SHOP_CONFIG::get_tax_rate($products_data['products_tax_class_id'])
      );
      $this->add_item_data('g:tax', $tax_array);
    }
    
    if (!AGI_EXPORT_QUICK) {
      if (!empty($products_data['agi_highlight_text'])) {
        $this->add_item_data('g:product_highlight', $products_data['agi_highlight_text']);
      }
      
      if (!empty($products_data['products_weight']) && round($products_data['products_weight'],2) > 0.01) {
        $this->add_item_data('g:shipping_weight', number_format($products_data['products_weight'], 2, ',', ''). ' kg');
      }
      
      if (!empty($products_data['agi_energy_label'])) {
        $this->add_item_data('g:energy_efficiency_class', $products_data['agi_energy_label']);
        $this->add_item_data('g:min_energy_efficiency_class', $products_data['agi_energy_label_sfrom']);
        $this->add_item_data('g:max_energy_efficiency_class', $products_data['agi_energy_label_sto']);
      }
      
      if (!empty($products_data['agi_export_length'])) {
        $this->add_item_data('g:product_length', $products_data['agi_export_length'].' cm');
      }
      if (!empty($products_data['agi_export_width'])) {
        $this->add_item_data('g:product_width', $products_data['agi_export_width'].' cm');
      }
      if (!empty($products_data['agi_export_height'])) {
        $this->add_item_data('g:product_height', $products_data['agi_export_height'].' cm');
      }
      
      if ($products_data['products_fsk18']) {
        $this->add_item_data('g:adult', 'TRUE');
      }
      
      if ($products_data['agi_bundle_amount'] > 1) {
        $this->add_item_data('g:multipack', $products_data['agi_bundle_amount']);
      }
      if ($products_data['agi_product_is_a_set'] == 1) {
        $this->add_item_data('g:is_bundle', 'TRUE');
      }
    }
    
    if (!empty($products_data['vpe'])) {
      $this->add_item_data('g:unit_pricing_measure', $products_data['unit_pricing_measure']);
      $this->add_item_data('g:unit_pricing_base_measure', $products_data['unit_pricing_base_measure']);
    }
    
    if (!AGI_EXPORT_QUICK) {
      for ($i=1; $i<=5; $i++) {
        if (!empty($products_data['agi_custom_label_'.$i])) {
          $this->add_item_data('g:custom_label_'.($i-1), $this->short_string($products_data['agi_custom_label_'.$i], 100));
        }
      }
      
      if (array_key_exists('agi_google_excluded_destination', $products_data) && !empty($products_data['agi_google_excluded_destination'])) {
        $tmp = explode(',', $products_data['agi_google_excluded_destination']);
        if (count($tmp) > 1) {
          $this->add_item_data('g:excluded_destination', $tmp);
        } else {
          $this->add_item_data('g:excluded_destination', $tmp[0]);
        }
      }
      if (array_key_exists('agi_google_included_destination', $products_data) && !empty($products_data['agi_google_included_destination'])) {
        $tmp = explode(',', $products_data['agi_google_included_destination']);
        if (count($tmp) > 1) {
          $this->add_item_data('g:included_destination', $tmp);
        } else {
          $this->add_item_data('g:included_destination', $tmp[0]);
        }
      }
      
      if (array_key_exists('agi_expirydate', $products_data) && !empty($products_data['agi_expirydate']) && $products_data['agi_expirydate'] != '0000-00-00') {
        $this->add_item_data('g:expiration_date', $products_data['agi_expirydate']);
      }
    
      if ($products_data['agi_is_clothing_product']) {
        if (array_key_exists('agi_attribute_color', $products_data) && !empty($products_data['agi_attribute_color'])) {
          $this->add_item_data('g:color', str_replace('/', '&#47;', $this->short_string($products_data['agi_attribute_color'], 100)));
        }
        elseif(array_key_exists('pvariant_attribute_color', $products_data) && !empty($products_data['pvariant_attribute_color'])) {
          $this->add_item_data('g:color', str_replace('/', '&#47;', $this->short_string($products_data['pvariant_attribute_color'], 100)));
        }
        
        if (array_key_exists('agi_attribute_size', $products_data) && !empty($products_data['agi_attribute_size'])) {
          $this->add_item_data('g:size', $this->short_string($products_data['agi_attribute_size'], 100));
        }
        elseif(array_key_exists('pvariant_attribute_size', $products_data) && !empty($products_data['pvariant_attribute_size'])) {
          $this->add_item_data('g:size', $this->short_string($products_data['pvariant_attribute_size'], 100));
        }
        else {
          $this->add_item_data('g:size', 'one size', 100);
        }
        
        if (array_key_exists('agi_attribute_pattern', $products_data) && !empty($products_data['agi_attribute_pattern'])) {
          $this->add_item_data('g:pattern', $this->short_string($products_data['agi_attribute_pattern'], 100));
        }
        elseif(array_key_exists('pvariant_attribute_pattern', $products_data) && !empty($products_data['pvariant_attribute_pattern'])) {
          $this->add_item_data('g:pattern', $this->short_string($products_data['pvariant_attribute_pattern'], 100));
        }
        
        if (array_key_exists('agi_attribute_material', $products_data) && !empty($products_data['agi_attribute_material'])) {
          $this->add_item_data('g:material', str_replace('/', '&#47;', $this->short_string($products_data['agi_attribute_material'], 200)));
        }
        elseif(array_key_exists('pvariant_attribute_material', $products_data) && !empty($products_data['pvariant_attribute_material'])) {
          $this->add_item_data('g:material', str_replace('/', '&#47;', $this->short_string($products_data['pvariant_attribute_material'], 200)));
        }
        
        if (array_key_exists('agi_export_gender', $products_data) && !empty($products_data['agi_export_gender'])) {
          $this->add_item_data('g:gender', $products_data['agi_export_gender']);
        }
        elseif(array_key_exists('pvariant_attribute_gender', $products_data) && !empty($products_data['pvariant_attribute_gender'])) {
          if (AGI_SHOP_CONFIG::is_assigned_word('gender', 'male', $products_data['pvariant_attribute_gender'])) {
            $this->add_item_data('g:gender', 'male');
          } elseif (AGI_SHOP_CONFIG::is_assigned_word('gender', 'female', $products_data['pvariant_attribute_gender'])) {
            $this->add_item_data('g:gender', 'female');
          } else {
            $this->add_item_data('g:gender', 'unisex');
          }
        }
        else {
          $this->add_item_data('g:gender', AGI_SHOP_CONFIG::get_agi_export_conf('preselect_gender'));
        }
          
        if(array_key_exists('agi_export_agegroup', $products_data) && !empty($products_data['agi_export_agegroup'])) {
          $this->add_item_data('g:age_group', $products_data['agi_export_agegroup']);
        }
        elseif (array_key_exists('pvariant_attribute_agegroup', $products_data) && !empty($products_data['pvariant_attribute_agegroup'])) {
          if (AGI_SHOP_CONFIG::is_assigned_word('age_group', 'adult', $products_data['pvariant_attribute_agegroup'])) {
            $this->add_item_data('g:age_group', 'adult');
          } elseif (AGI_SHOP_CONFIG::is_assigned_word('age_group', 'kids', $products_data['pvariant_attribute_agegroup'])) {
            $this->add_item_data('g:age_group', 'kids');
          } elseif (AGI_SHOP_CONFIG::is_assigned_word('age_group', 'newborn', $products_data['pvariant_attribute_agegroup'])) {
            $this->add_item_data('g:age_group', 'newborn');
          } elseif (AGI_SHOP_CONFIG::is_assigned_word('age_group', 'infant', $products_data['pvariant_attribute_agegroup'])) {
            $this->add_item_data('g:age_group', 'infant');
          } elseif (AGI_SHOP_CONFIG::is_assigned_word('age_group', 'toddler', $products_data['pvariant_attribute_agegroup'])) {
            $this->add_item_data('g:age_group', 'toddler');
          } else {
            $this->add_item_data('g:age_group', 'adult');
          }
        }
        else {
          $this->add_item_data('g:age_group', AGI_SHOP_CONFIG::get_agi_export_conf('preselect_agegroup'));
        }
        
        if (!empty($products_data['agi_size_type'])) {
          $this->add_item_data('g:size_type', $products_data['agi_size_type']);
        }
        else {
          $this->add_item_data('g:size_type', AGI_SHOP_CONFIG::get_agi_export_conf('preselect_sizetype'));
        }
        if (!empty($products_data['agi_size_system'])) {
          $this->add_item_data('g:size_system', $products_data['agi_size_system']);
        }
        else {
          $this->add_item_data('g:size_system', AGI_SHOP_CONFIG::get_agi_export_conf('clothes_sizesystem'));
        }
      }
      else {
        if (array_key_exists('agi_attribute_color', $products_data) && !empty($products_data['agi_attribute_color'])) {
          $this->add_item_data('g:color', str_replace('/', '&#47;', $this->short_string($products_data['agi_attribute_color'], 100)));
        } elseif(array_key_exists('pvariant_attribute_color', $products_data) && !empty($products_data['pvariant_attribute_color'])) {
          $this->add_item_data('g:color', str_replace('/', '&#47;', $this->short_string($products_data['pvariant_attribute_color'], 100)));
        }
        if (array_key_exists('agi_attribute_size', $products_data) && !empty($products_data['agi_attribute_size'])) {
          $this->add_item_data('g:size', $this->short_string($products_data['agi_attribute_size'], 100));
        } elseif(array_key_exists('pvariant_attribute_size', $products_data) && !empty($products_data['pvariant_attribute_size'])) {
          $this->add_item_data('g:size', $this->short_string($products_data['pvariant_attribute_size'], 100));
        }
        if (array_key_exists('agi_attribute_pattern', $products_data) && !empty($products_data['agi_attribute_pattern'])) {
          $this->add_item_data('g:pattern', $this->short_string($products_data['agi_attribute_pattern'], 100));
        } elseif(array_key_exists('pvariant_attribute_pattern', $products_data) && !empty($products_data['pvariant_attribute_pattern'])) {
          $this->add_item_data('g:pattern', $this->short_string($products_data['pvariant_attribute_pattern'], 100));
        }
        if (array_key_exists('agi_attribute_material', $products_data) && !empty($products_data['agi_attribute_material'])) {
          $this->add_item_data('g:material', str_replace('/', '&#47;', $this->short_string($products_data['agi_attribute_material'], 200)));
        } elseif(array_key_exists('pvariant_attribute_material', $products_data) && !empty($products_data['pvariant_attribute_material'])) {
          $this->add_item_data('g:material', str_replace('/', '&#47;', $this->short_string($products_data['pvariant_attribute_material'], 200)));
        }
        if (array_key_exists('agi_export_gender', $products_data) && !empty($products_data['agi_export_gender'])) {
          $this->add_item_data('g:gender', $products_data['agi_export_gender']);
        }
        if(array_key_exists('agi_export_agegroup', $products_data) && !empty($products_data['agi_export_agegroup'])) {
          $this->add_item_data('g:age_group', $products_data['agi_export_agegroup']);
        }
      }
      
      if ($products_data['agi_ekp'] > 0) {
        $this->add_item_data('purchase_price', AGI_SHOP_CONFIG::add_tax($products_data['agi_ekp'], AGI_SHOP_CONFIG::get_tax_rate($products_data['products_tax_class_id'])));
      }
    }
  }
}
